/*

DESCRIPTION:
    Main source file for SSR reference design.

MPU:  PIC12F1612

SYNTAX:  n/a

KNOWN BUGS:  n/a

COPYRIGHT:  (c) 2018, Brian Cornell, all rights reserved.

DEPENDENCIES:
    config.h - PIC configuration registers
    main.h - configuration constants for main.c

VERSION:  2.1

MODIFICATION HISTORY:

    11 May 18, BWC, released 2.1
    1 Mar 18, BWC, released 2.0
    26 Feb 18, BWC, created file.
    

*/

//Config PIC registers first
#include "config.h"

//System includes second
#include <pic.h>
#include <xc.h>
#include <stdint.h>


//User includes last
#include <binary.h>
#include "main.h"


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                              Data objects
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#ifdef FEATURE_NTC
bit tempHOT, tempCHK;                       //Flags to indicate temperature status
#endif


/*Function main.
 *
 * Returns:  none
 *
 * Parameters:  none
 *
*/
void main(void)
{
    unsigned char timer = 0;
#ifdef FEATURE_NTC
    uint16_t hysHOT, hysOVH;                //Temp sense hysteresis counters

    tempHOT = FALSE;
    tempCHK = S_BL_HOT;
    hysHOT = 0;
    hysOVH = 0;
#endif

    Initialize();

#ifdef FEATURE_NTC
    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * If temp feature enabled do a one-time check (e.g. without hysteresis) for
     * high temp to set tempHOT TRUE.  Better to prevent a start right after 
     * boot than allow only to immediately shut down.
     +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    if (S_MP_HOT) tempHOT = TRUE;
#endif

    while (TRUE)
    {
        CLRWDT();                           //Reset watchdog timer

#ifdef FEATURE_NTC
        /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
         * Monitor SSR temp if feature enabled.  Alternate between HOT and
         * OverHeat checks.
         * 
         * Behavior.
         * 1) If controller boots with temperature above HOT threshold it
         * will not start.  Likewise, if already on and temperature is above HOT
         * threshold prior to CTRL set TRUE SSR will not turn on.
         * 2) When running and temperature rises above HOT threshold the ACK
         * pin is set FALSE but SSR remains on.
         * 3) When temperature rises above OVH threshold the SSR is turned off,
         * and will not resume operation, provided CTRL remains TRUE, until 
         * temperature falls below HOT threshold.
         * 
         * The Digital to Analog Converter (DAC) and Comparator are used for
         * temperature monitoring.  The NTC is expected to be the bottom 
         * resistor in the voltage divider so as temperature rises the voltage
         * on the Comparator's (-) pin will fall.
         * 
         * During normal operation the DAC is set to the HOT threshold and the
         * comparator's output monitored.  When TRUE the algorithm monitors
         * both the HOT and OVH thresholds by toggling the DAC's value.
         * 
         * Hysteresis is employed for noise filtering.  Successive samples
         * increment the hysteresis counter but any gap in positive samples 
         * decrements thereby extending the required sample time to confirm.
         +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        if (tempHOT)
        {
            if (tempCHK == S_BL_OVH)        //Check for overheating
            {
                if (S_MP_OVH)
                {
                    if (++hysOVH > S_LM_OVH_HYS)
                    {
                        pwmOFF();           //Overheat threshold confirmed:  turn SSR off
                        hysOVH = 0;
                    }
                }
                else if (hysOVH) --hysOVH;

                if (!(hysOVH))              //Toggle to HOT temp check
                {
                    tempCHK = S_BL_HOT;
                    SETDACHOT();
                }
            }
            else                            //Check if SSR has cooled down
            {
                if (!(S_MP_HOT))
                {
                    if (++hysHOT > S_LM_HOT_HYS)
                    {
                        tempHOT = FALSE;    //Confirmed, resume operation
                        hysHOT = 0;
                        if (!(S_MP_SSR_OFF)) SETACKON();    //Clear fault
                    }
                }
                else if (hysHOT) --hysHOT;

                //Only check for overheating if SSR is still on
                if (!(hysHOT) && (tempHOT) && !(S_MP_SSR_OFF))
                {
                    tempCHK = S_BL_OVH;
                    SETDACOVH();
                }
            }
        }
        else
        {
            if (S_MP_HOT)                   //Hot temp check
            {
                if (++hysHOT > S_LM_HOT_HYS)
                {
                    hysHOT = 0;             //Confirmed, next check for overheat
                    tempHOT = TRUE;
                    tempCHK = S_BL_OVH;
                    SETDACOVH();
                    SETACKOFF();            //If on keep running but signal high temp
                }
            }
            else if (hysHOT) --hysHOT;
        }
#endif

        /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
         *                        Turn SSR ON...
         +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#ifdef FEATURE_NTC
        //Ignore if already hot
        if (((PIN_I_CTRL) && (S_MP_SSR_OFF)) && !(tempHOT))
#else
        if ((PIN_I_CTRL) && (S_MP_SSR_OFF))
#endif
        {
            //...after timeout period to ensure no false triggers
            if (++timer > S_TM_TIMEOUT)
            {
                pwmON();
                timer = 0;                  //Clear timer
            }
            else continue;
        }

        /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
         *                       Turn SSR OFF...
         +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        if (!(PIN_I_CTRL) && !(S_MP_SSR_OFF))
        {
            //...after timeout period to ensure no false triggers
            if (++timer > S_TM_TIMEOUT) pwmOFF();
            else continue;
        }

        //If bottom of loop reached then no timing in progress so clear
        timer = 0;
    }
} //End function main


/*Function pwmON.  Turns SSR on.  Duty Cycle is initially boosted for hard turn-on.
 *
 * Returns:  none
 *
 * Parameters:  none
 *
*/
void pwmON(void)
{
    //Disengage active turn-off if present
#ifdef PIN_O_OFF
    PIN_O_OFF = FALSE;
#endif

    //Load starting DC
    CCPR1H = (CFG_CCP_1_DC_START >> 8);
    CCPR1L = (uint8_t) (0x00FF & CFG_CCP_1_DC_START);

    //Turn MOSFET(s) on
    S_MP_SSR_OFF = FALSE;
#ifdef FEATURE_DUAL_DIRECT
    PIN_O_POLE2 = TRUE;
#endif

    //Ensure a few cycles run at boost level for hard turn-on
    __delay_us(S_TM_ONWAIT);

    //Load steady state DC
    CCPR1H = (CFG_CCP_1_DC_RUN >> 8);
    CCPR1L = (uint8_t) (0x00FF & CFG_CCP_1_DC_RUN);

    SETACKON();                             //Acknowledge turn-on

    return;
}


/*Function pwmOFF.  Turns SSR off.
 *
 * Returns:  none
 *
 * Parameters:  none
 *
*/
void pwmOFF(void)
{
    //SSR off
    S_MP_SSR_OFF = TRUE;
#ifdef FEATURE_DUAL_DIRECT
    PIN_O_POLE2 = FALSE;
#endif

    //Engage active turn-off if present
#ifdef PIN_O_OFF
    PIN_O_OFF = TRUE;
#endif

#ifdef FEATURE_ACTIVE_TURNOFF
    //Pause and then disengage active turn-off
    __delay_us(S_TM_OFFWAIT);
    PIN_O_OFF = FALSE;
#endif

    SETACKOFF();                            //Acknowledge turn-off

    return;
}


/*Function Initialize.  Performs baseline MPU initialization at boot.
 *
 * Returns:  none
 *
 * Parameters:  none
 *
*/
void Initialize(void)
{
    CLRWDT();                           //Reset WDT
    di();                               //No interrupts

    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * Configure system clock & watch dog timer
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    OSCCON = CFG_OSC_CON;


    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * Various configuration settings to disable defaults that prevent port/pin
     * access.  IMPORTANT:  these must be set prior to port configuration.
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    OPTION_REG = CFG_OPT_REG;
    T1CON = CFG_TMR_1_CON;

    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * Set port configuration.  To prevent any aberrant behaviors before 
     * initialization is complete set output latches.
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    //Port A
    PORTA = VL_PORTA;
    TRISA = IO_PORTA;
    ANSELA = AD_PORTA;
    WPUA = WP_PORTA;
    ODCONA = OD_PORTA;
    SLRCONA = SR_PORTA;
    INLVLA = IL_PORTA;


    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * Configure timer2 & CCP (PWM)
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    T2CLKCON = CFG_TMR_2_CLK;
    T2CON = CFG_TMR_2_CON;
    PR2 = CFG_TMR_2_PR2;
    CCPTMRS = CFG_CCP_TMRS;
    CCP1CON = CFG_CCP_1_CON;


    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * Configure CWG:  Auto-shutdown must be configured first to prevent 
     * un-commanded turn-on of SSR.
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    CWG1AS0 = CFG_CWG_1_AS0;
    CWG1AS1 = CFG_CWG_1_AS1;
    CWG1OCON0 = CFG_CWG_1_OCON0;
    CWG1OCON1 = CFG_CWG_1_OCON;
    CWG1CLKCON = CFG_CWG_1_CLK;
    CWG1ISM = CFG_CWG_1_ISM;
    CWG1CON0 = CFG_CWG_1_CON0;
    CWG1CON1 = CFG_CWG_1_CON1;


#ifdef FEATURE_NTC
    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * Configure NTC temperature sensing if feature enabled
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    //DAC
    DAC1CON0 = CFG_DAC_CON0;
    SETDACHOT();

    //Comparator
    CM1CON0 = CFG_CM_CON0;
    CM1CON1 = CFG_CM_CON1;
#endif


    TMR2ON = TRUE;                          //Start PWM/CWG but no output


    return;
} //End function Initialize



//End of file