/*

DESCRIPTION:
    Main include file for SSR.

MPU:  PIC12F1612

SYNTAX:  n/a

KNOWN BUGS:  n/a.

COPYRIGHT:  (c) 2018, Brian Cornell, all rights reserved.

DEPENDENCIES:  none

VERSION:  2.1

MODIFICATION HISTORY:

    11 May 18, BWC, released 2.1
    1 Mar 18, BWC, released 2.0
    26 Feb 18, BWC, created file.
 

*/



//#define _DEBUG                              //Uncomment for conditional testing


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                      Version constants & definitions
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define VERMAJOR                2
#define VERMINOR                1
#define VERBUILD                7


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Conditional compile features.  Uncomment each #define below to enable a 
 * feature.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
//Use active turn-off but disengage when done
//#define FEATURE_ACTIVE_TURNOFF

//Use active turn-off and keep engaged while off - good for high noise immunity
//#define FEATURE_ACTIVE_OFF

//Use higher duty cycle for start:  reduces SSR turn-on time
//#define FEATURE_DC_START_HIGH

//Use higher duty cycle for steady-state drive:  recommended if no gate block diode
#define FEATURE_DC_RUN_HIGH

//Dual drive:  for split SSR with each drive at a different potential
#define FEATURE_DUAL_SSR

//Dual drive direct:  the second pole of relay is ground referenced and driven
//with steady logic '1' (TRUE) instead of PWM.
#define FEATURE_DUAL_DIRECT

//Thermal protection using NTC thermistor.  This feature is mutually exclusive ACTIVE_XXX
#define FEATURE_NTC


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * System wide constants & definitions.  Convention:  S_<TYPE>_<INFO>
 * Types:
 *  + DC=Duty Cycle
 *  + EN=ENable bit
 *  + RT=RaTio of a voltage presented to PIC, internal or external
 *  + LM=LiMiting operational value
 *  + TM=TiMing
 *  + MP=Peripheral port bit MaPping
 *  + QY=quantity (count) of something
 *  + CD=command data
 *  + BL=boolean value
 * 
 * Info is user defined and should be descriptive.
 * 
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define S_TM_TIMEOUT        13              //# of loops to delay SSR turn-on/off
#define S_TM_ONWAIT         20              //#uS to wait before change to steady state DC
#define S_TM_OFFWAIT        20              //#uS to wait before disengaging turn-off circuit
#define S_MP_HOT            C1OUT           //Comparator 1 output bit
#define S_MP_OVH            C1OUT           //Comparator 1 output bit
#define S_TM_DACCHG         50              //#uS for DAC/Comparator to settle after DAC change
#define S_LM_HOT_HYS        100             //Hysteresis for hot temperature threshold
#define S_LM_OVH_HYS        100             //Hysteresis for overheattemperature threshold
#define S_BL_HOT            FALSE           //Checking for HOT temp
#define S_BL_OVH            TRUE            //Checking for OverHeat temp
#define S_MP_SSR_OFF        CWG1SHUTDOWN    //Indicates & sets SSR on/off


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Analog channel mapping:  channel=port=pin.  The values assigned to these
 * constants should match those used by ADC and bit-aligned for direct loading
 * into ADC control register.  Channels available depend on specific chip.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                           PPS & direct pin mapping
 * 
 * For PPS output pin assignments both aliases for pin registers and constants 
 * mapping what peripheral connects to the pin.
 *      Pin register alias convention:  PPS_P_XXX_YY where
 *          XXX=peripheral name
 *          YY=channel or other detail; can be omitted
 *      Peripheral mapping convention:  PPS_M_XXX_YY same as alias
 * 
 * Input pins only require mapping constant.
 * 
 * For pins accessed directly or assigned in a peripheral's register (like ADC)
 * use the following convention:  PIN_X_YYYYYY where
 *          X=Input or Output
 *          YYYYYY=user assigned mnemonic describing function
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define PIN_I_CTRL              RA5         //TRUE=SSR on
#define PIN_O_ACK               RA4         //TRUE=controller acknowledges PWM is ON
#define PIN_O_POLE2             RA0         //Second pole direct drive pin


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * DO NOT EDIT:  controls feature compile
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#if (defined FEATURE_ACTIVE_TURNOFF || defined FEATURE_ACTIVE_OFF)
#define PIN_O_OFF               RA1         //Drives SSR hard turn-off circuit
#endif
#if defined(PIN_O_OFF) && defined(FEATURE_NTC)
#error "FEATURE_NTC is mutually exclusive with FEATURE_ACTIVE_XOFF"
#endif


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Port/pin configuration:  I/O, analog/digital, slew rate, etc.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#ifndef PIN_O_OFF
#define _IO_PORTA               0b00101011  //1=input, 0=output
#define _AD_PORTA               0b00000011  //1=analog, 0=digital
#define WP_PORTA                0b00000000  //Weak pull-up:  1=enabled, 0=disabled
#else
#define _IO_PORTA               0b00101001  //1=input, 0=output
#define _AD_PORTA               0b00000001  //1=analog, 0=digital
#define WP_PORTA                0b00000000  //Weak pull-up:  1=enabled, 0=disabled
#endif

//Hack to enable pin for dual SSR feature
#ifdef FEATURE_DUAL_SSR
#define IO_PORTA                (_IO_PORTA & 0xFE)
#define AD_PORTA                (_AD_PORTA & 0xFE)
#else
#define IO_PORTA                _IO_PORTA
#define AD_PORTA                _AD_PORTA
#endif

#ifdef FEATURE_ACTIVE_OFF
#define VL_PORTA                0b00000010  //Initial pin values
#else
#define VL_PORTA                0b00000000  //Initial pin values
#endif

#define OD_PORTA                0b00000000  //Open drain:  1=open drain, 0=push-pull
#define SR_PORTA                0b00000000  //Slew rate:  1=limited, 0=maximum
#define IL_PORTA                0b00000000  //Input level:  1=Schmitt, 0=TTL


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                            Macro functions
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define SETDACHOT()         DAC1CON1 = CFG_DAC_HOT
#define SETDACOVH()         DAC1CON1 = CFG_DAC_OVH
#define SETACKOFF()         PIN_O_ACK = FALSE
#define SETACKON()          PIN_O_ACK = TRUE


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Various configuration settings to disable defaults that prevent port/pin
 * access.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
//Timer0 & options register
#define CFG_OPT_REG         0x00            //WPU indv control, Fosc/4, use PSA, 1:2 prescale
//Timer1
#define CFG_TMR_1_CON       0x00            //Timer1 clock source is internal


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Timer2 configuration constants
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_TMR_2_CLK       0x00            //Fosc/4
#define CFG_TMR_2_CON       0x00            //1:1 post & prescale, timer off
#define CFG_TMR_2_PR2       0x09            //Timer2 period value


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * CCP1 configuration constants - PWM
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_CCP_1_CON       0x8C            //Enabled, DC right-aligned, PWM mode
#define CFG_CCP_TMRS        0x00            //CCP1 & 2 use timer2

//Steady state duty cycle varies depending on HW config of SSR
#ifdef FEATURE_DC_RUN_HIGH
#define CFG_CCP_1_DC_RUN    0x0004          //No blocking diode @ gate
#else
#define CFG_CCP_1_DC_RUN    0x0003          //Blocking diode @ gate
#endif

//Start-up duty cycle
#ifdef FEATURE_DC_START_HIGH
#define CFG_CCP_1_DC_START  0x0005          //Boost to charge MOSFET gates quickly
#else
#define CFG_CCP_1_DC_START  CFG_CCP_1_DC_RUN
#endif


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * CWG (Complimentary Waveform Generator) configuration:  used to steer PWM
 * output to second pin when driving a dual (or split) SSR
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_CWG_1_CON0      0x80            //Enabled, steering mode
#define CFG_CWG_1_CON1      0x00            //Normal polarity
#define CFG_CWG_1_AS0       0x80            //Auto-shutdown, use inactive state
#define CFG_CWG_1_AS1       0x00            //All HW auto-shutdown sources disabled
#define CFG_CWG_1_OCON0     0x0F            //Data waveform & polarity bits control output
#define CFG_CWG_1_CLK       0x00            //Clock is Fosc
#define CFG_CWG_1_ISM       0x03            //CCP1 is input to CWG

#if defined(FEATURE_DUAL_SSR) && !defined(FEATURE_DUAL_DIRECT)
#define CFG_CWG_1_OCON      0x03            //Enable A & B outputs of CWG
#else
#define CFG_CWG_1_OCON      0x01            //Enable A output of CWG
#endif

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * ADC configuration & acquisition constants
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
//Internal channels (peripherals)


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Configure system clock.  Expects 4MHz HFINTOSC to be active at boot.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_OSC_CON         0x68            //4MHz system clock
#define _XTAL_FREQ          4000000         //Must match Fosc to use __delayxxx macros


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * DAC configuration constants
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_DAC_CON0        0x80            //Enabled, Vref+=Vdd
#define CFG_DAC_HOT         0x67            //Hot threshold:  2V with Vref=5V
#define CFG_DAC_OVH         0x4D            //Overheat threshold:  1.5V with Vref=5V


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Comparator configuration constants
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_CM_CON0         0x82            //Enable, low speed, hysteresis
#define CFG_CM_CON1         0x10            //+ = DAC, - = AN1


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Watchdog configuration constants
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Function declarations.  See source file for explanation of each function.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void main(void);
void pwmON(void);
void pwmOFF(void);
void Initialize(void);


//End of file